@@ -26,7 +26,7 @@
 
 #include <gtk/gtkeventbox.h>
 
-#include <libempathy/empathy-avatar.h>
+#include <libempathy/empathy-contact.h>
 
 G_BEGIN_DECLS
 
@@ -42,6 +42,7 @@ typedef struct _EmpathyAvatarImageClass EmpathyAvatarImageClass;
 
 struct _EmpathyAvatarImage {
 	GtkEventBox parent;
+	gpointer priv;
 };
 
 struct _EmpathyAvatarImageClass {
