@@ -34,12 +34,10 @@
 #include "empathy-theme-irc.h"
 #include "empathy-theme-manager.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), EMPATHY_TYPE_THEME_MANAGER, EmpathyThemeManagerPriv))
-
+#define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyThemeManager)
 typedef struct {
 	gchar       *name;
 	guint        name_notify_id;
-	guint        room_notify_id;
 
 	gboolean     show_avatars;
 	guint        show_avatars_notify_id;
@@ -56,9 +54,7 @@ static void        theme_manager_finalize                 (GObject            *o
 static void        theme_manager_notify_name_cb           (EmpathyConf         *conf,
 							   const gchar        *key,
 							   gpointer            user_data);
-static void        theme_manager_notify_room_cb           (EmpathyConf         *conf,
-							   const gchar        *key,
-							   gpointer            user_data);
+
 static void        theme_manager_notify_show_avatars_cb   (EmpathyConf         *conf,
 							   const gchar        *key,
 							   gpointer            user_data);
@@ -214,22 +210,16 @@ empathy_theme_manager_class_init (EmpathyThemeManagerClass *klass)
 static void
 empathy_theme_manager_init (EmpathyThemeManager *manager)
 {
-	EmpathyThemeManagerPriv *priv;
-
-	priv = GET_PRIV (manager);
+	EmpathyThemeManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
+		EMPATHY_TYPE_THEME_MANAGER, EmpathyThemeManagerPriv);
 
+	manager->priv = priv;
 	priv->name_notify_id =
 		empathy_conf_notify_add (empathy_conf_get (),
 					EMPATHY_PREFS_CHAT_THEME,
 					theme_manager_notify_name_cb,
 					manager);
 
-	priv->room_notify_id =
-		empathy_conf_notify_add (empathy_conf_get (),
-					EMPATHY_PREFS_CHAT_THEME_CHAT_ROOM,
-					theme_manager_notify_room_cb,
-					manager);
-
 	empathy_conf_get_string (empathy_conf_get (),
 				EMPATHY_PREFS_CHAT_THEME,
 				&priv->name);
@@ -289,7 +279,6 @@ theme_manager_finalize (GObject *object)
 	priv = GET_PRIV (object);
 
 	empathy_conf_notify_remove (empathy_conf_get (), priv->name_notify_id);
-	empathy_conf_notify_remove (empathy_conf_get (), priv->room_notify_id);
 	empathy_conf_notify_remove (empathy_conf_get (), priv->show_avatars_notify_id);
 
 	g_free (priv->name);
@@ -328,14 +317,6 @@ theme_manager_notify_name_cb (EmpathyConf  *conf,
 	g_signal_emit (manager, signals[THEME_CHANGED], 0, NULL);
 }
 
-static void
-theme_manager_notify_room_cb (EmpathyConf  *conf,
-			      const gchar *key,
-			      gpointer     user_data)
-{
-	g_signal_emit (user_data, signals[THEME_CHANGED], 0, NULL);
-}
-
 static void
 theme_manager_notify_show_avatars_cb (EmpathyConf  *conf,
 				      const gchar *key,
