@@ -8,12 +8,12 @@
 
 G_BEGIN_DECLS
 
+#include <libempathy/empathy-debug.h>
+#define EMPATHY_TYPE_DEBUG_FLAGS empathy_debug_flags_get_type()
+GType empathy_debug_flags_get_type (void);
 #include <libempathy/empathy-utils.h>
 #define EMPATHY_TYPE_REG_EX_TYPE empathy_reg_ex_type_get_type()
 GType empathy_reg_ex_type_get_type (void);
-#include <libempathy/empathy-message.h>
-#define EMPATHY_TYPE_MESSAGE_TYPE empathy_message_type_get_type()
-GType empathy_message_type_get_type (void);
 #include <libempathy/empathy-contact.h>
 #define EMPATHY_TYPE_CAPABILITIES empathy_capabilities_get_type()
 GType empathy_capabilities_get_type (void);
