@@ -38,7 +38,7 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 
-typedef struct 
+typedef struct
 {
   EmpathyTpCall *call;
   GTimeVal start_time;
@@ -61,6 +61,8 @@ typedef struct
   GtkWidget *keypad_expander;
 } EmpathyCallWindow;
 
+static GSList *windows = NULL;
+
 static gboolean
 call_window_update_timer (gpointer data)
 {
@@ -269,8 +271,11 @@ call_window_destroy_cb (GtkWidget *widget,
                         EmpathyCallWindow *window)
 {
   call_window_finalize (window);
+
   g_object_unref (window->output_video_socket);
   g_object_unref (window->preview_video_socket);
+
+  windows = g_slist_remove (windows, window);
   g_slice_free (EmpathyCallWindow, window);
 }
 
@@ -461,17 +466,48 @@ call_window_dtmf_connect (GladeXML *glade,
 }
 
 GtkWidget *
-empathy_call_window_new (EmpathyTpCall *call)
+empathy_call_window_new (TpChannel *channel)
 {
   EmpathyCallWindow *window;
   GladeXML *glade;
   gchar *filename;
   const gchar *icons[] = {"audio-input-microphone", NULL};
 
-  g_return_val_if_fail (EMPATHY_IS_TP_CALL (call), NULL);
+  g_return_val_if_fail (TP_IS_CHANNEL (channel), NULL);
+
+  if (windows)
+    {
+      window = (EmpathyCallWindow*) windows->data;
+      if (!window->call)
+        {
+          window->call = empathy_tp_call_new (channel);
+          g_signal_connect_swapped (G_OBJECT (window->call), "notify",
+              G_CALLBACK (call_window_update), window);
+          call_window_update (window);
+        }
+      else
+        {
+          GtkWidget *dialog;
+
+          /* We don't want to have multiple calls running.
+           * FIXME: We should use the hold interface... */
+          tp_cli_channel_call_close (channel, -1, NULL, NULL, NULL, NULL);
+
+          dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO,
+              GTK_BUTTONS_CLOSE,
+              _("Call rejected because there is already a running call."));
+          g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy),
+              NULL);
+          gtk_widget_show (dialog);
+        }
+
+      gtk_window_present (GTK_WINDOW (window->window));
+      return window->window;
+    }
 
   window = g_slice_new0 (EmpathyCallWindow);
-  window->call = g_object_ref (call);
+  windows = g_slist_prepend (windows, window);
+  window->call = empathy_tp_call_new (channel);
 
   filename = empathy_file_lookup ("empathy-call-window.glade", "src");
   glade = empathy_glade_get_file (filename,
