@@ -39,7 +39,6 @@
 #include <libempathy/empathy-tp-roomlist.h>
 #include <libempathy/empathy-chatroom.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-debug.h>
 
 #include <libempathy-gtk/empathy-account-chooser.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
@@ -47,7 +46,8 @@
 #include "empathy-new-chatroom-dialog.h"
 #include "ephy-spinner.h"
 
-#define DEBUG_DOMAIN "NewChatroomDialog"
+#define DEBUG_FLAG EMPATHY_DEBUG_OTHER
+#include <libempathy/empathy-debug.h>
 
 typedef struct {
 	EmpathyTpRoomlist *room_list;
@@ -95,7 +95,7 @@ static void     new_chatroom_dialog_new_room_cb                     (EmpathyTpRo
 								     EmpathyChatroom          *chatroom,
 								     EmpathyNewChatroomDialog *dialog);
 static void     new_chatroom_dialog_listing_cb                      (EmpathyTpRoomlist        *room_list,
-								     gboolean                  listing,
+								     gpointer                  unused,
 								     EmpathyNewChatroomDialog *dialog);
 static void     new_chatroom_dialog_model_clear                     (EmpathyNewChatroomDialog *dialog);
 static void     new_chatroom_dialog_model_row_activated_cb          (GtkTreeView             *tree_view,
@@ -366,7 +366,7 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 		g_signal_connect (dialog->room_list, "new-room",
 				  G_CALLBACK (new_chatroom_dialog_new_room_cb),
 				  dialog);
-		g_signal_connect (dialog->room_list, "listing",
+		g_signal_connect (dialog->room_list, "notify::listing",
 				  G_CALLBACK (new_chatroom_dialog_listing_cb),
 				  dialog);
 
@@ -397,9 +397,9 @@ new_chatroom_dialog_new_room_cb (EmpathyTpRoomlist        *room_list,
 	GtkListStore     *store;
 	GtkTreeIter       iter;
 
-	empathy_debug (DEBUG_DOMAIN, "New chatroom listed: %s (%s)",
-		       empathy_chatroom_get_name (chatroom),
-		       empathy_chatroom_get_room (chatroom));
+	DEBUG ("New chatroom listed: %s (%s)",
+		empathy_chatroom_get_name (chatroom),
+		empathy_chatroom_get_room (chatroom));
 
 	/* Add to model */
 	view = GTK_TREE_VIEW (dialog->treeview);
@@ -415,9 +415,13 @@ new_chatroom_dialog_new_room_cb (EmpathyTpRoomlist        *room_list,
 
 static void
 new_chatroom_dialog_listing_cb (EmpathyTpRoomlist        *room_list,
-				gboolean                  listing,
+				gpointer                  unused,
 				EmpathyNewChatroomDialog *dialog)
 {
+	gboolean listing;
+
+	listing = empathy_tp_roomlist_is_listing (room_list);
+
 	/* Update the throbber */
 	if (listing) {
 		ephy_spinner_start (EPHY_SPINNER (dialog->throbber));		
@@ -502,7 +506,7 @@ new_chatroom_dialog_join (EmpathyNewChatroomDialog *dialog)
 		room_name = g_strdup (room);
 	}
 
-	empathy_debug (DEBUG_DOMAIN, "Requesting channel for '%s'", room_name);
+	DEBUG ("Requesting channel for '%s'", room_name);
 
 	mc = empathy_mission_control_new ();
 	mission_control_request_channel_with_string_handle (mc,
